"""Common test configuration"""
import pytest
import responses


@pytest.fixture
def mocked_responses():
    """Mock requests responses"""
    with responses.RequestsMock() as rsps:
        yield rsps



"""Tests for our backend"""
from urllib.parse import urljoin

import pytest

from social_auth_mitxpro.backends import MITxProOAuth2


# pylint: disable=redefined-outer-name


@pytest.fixture
def strategy(mocker):
    """Mock strategy"""
    return mocker.Mock()


@pytest.fixture
def backend(strategy):
    """MITxProOAuth2 backend fixture"""
    return MITxProOAuth2(strategy)


@pytest.mark.parametrize(
    "response, expected",
    [
        (
            {"username": "abc123", "email": "user@example.com", "name": "Jane Doe"},
            {"username": "abc123", "email": "user@example.com", "name": "Jane Doe"},
        ),
        ({"username": "abc123"}, {"username": "abc123", "email": "", "name": ""}),
    ],
)
def test_get_user_details(backend, response, expected):
    """Test that get_user_details produces expected results"""
    assert backend.get_user_details(response) == expected


def test_user_data(backend, strategy, mocked_responses):
    """Tests that the backend makes a correct appropriate request"""
    access_token = "user_token"
    api_root = "http://xpro.example.com/"
    response = {"username": "abc123", "email": "user@example.com", "name": "Jane Doe"}

    mocked_responses.add(
        mocked_responses.GET, urljoin(api_root, "/api/users/me"), json=response
    )
    settings = {"API_ROOT": api_root}

    def _setting(name, *, backend, default=None):  # pylint: disable=unused-argument
        """Dummy setting func"""
        return settings.get(name, default)

    strategy.setting.side_effect = _setting

    assert backend.user_data(access_token) == response

    request, _ = mocked_responses.calls[0]

    assert request.headers["Authorization"] == "Bearer user_token"
    strategy.setting.assert_any_call("API_ROOT", default=None, backend=backend)


def test_authorization_url(backend, strategy):
    """Test authorization_url()"""
    strategy.setting.return_value = "abc"
    assert backend.authorization_url() == "abc"
    strategy.setting.assert_called_once_with(
        "AUTHORIZATION_URL", default=None, backend=backend
    )


def test_access_token_url(backend, strategy):
    """Test access_token_url()"""
    strategy.setting.return_value = "abc"
    assert backend.access_token_url() == "abc"
    strategy.setting.assert_called_once_with(
        "ACCESS_TOKEN_URL", default=None, backend=backend
    )



"""MIT xPro social auth backend"""
from social_core.backends.oauth import BaseOAuth2


class MITxProOAuth2(BaseOAuth2):
    """MIT xPro social auth backend"""

    name = "mitxpro-oauth2"

    ID_KEY = "username"
    REQUIRES_EMAIL_VALIDATION = False

    ACCESS_TOKEN_METHOD = "POST"

    # at a minimum we need to be able to read the user
    DEFAULT_SCOPE = ["user:read"]

    def authorization_url(self):
        """Provides authorization_url from settings"""
        return self.setting("AUTHORIZATION_URL")

    def access_token_url(self):
        """Provides access_token_url from settings"""
        return self.setting("ACCESS_TOKEN_URL")

    def api_root(self):
        """Returns the API root as configured"""
        root = self.setting("API_ROOT")

        if root and root[-1] != "/":
            root = f"{root}/"

        return root

    def auth_html(self):  # pragma: no cover
        """No html for this provider"""
        # NOTE: this is only here to stop the pylint warning about this abstract
        # method not being overridden without disabling it for the entire class
        return ""

    def api_url(self, path):
        """
        Returns the full api url given a relative path

        Args:
            path (str): relative api path
        """
        return f"{self.api_root()}{path}"

    def get_user_details(self, response):
        """Return user details from xPro account"""
        return {
            "username": response.get("username"),
            "email": response.get("email", ""),
            "name": response.get("name", ""),
        }

    def user_data(self, access_token, *args, **kwargs):
        """Loads user data from xpro"""
        url = self.api_url("api/users/me")
        headers = {"Authorization": f"Bearer {access_token}"}
        return self.get_json(url, headers=headers)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['social_auth_mitxpro']

package_data = \
{'': ['*']}

install_requires = \
['attrs==19.3.0', 'social-auth-core>=4.1.0,<5.0.0']

setup_kwargs = {
    'name': 'social-auth-mitxpro',
    'version': '0.5',
    'description': 'python-social-auth backend for mitxpro',
    'long_description': "\nsocial-auth-mitxpro\n---\n\n\n#### Prerequisites\n\n- [`pyenv`](https://github.com/pyenv/pyenv#installation) for managing python versions\n  - Install `python3.6` and `python2.7`\n- `pip install tox tox-pyenv` for running tests and discovering python versions from `pyenv`\n- [`poetry`](https://poetry.eustace.io/docs/#installation) for building, testing, and releasing\n\nIf this is your first time using `poetry`, you'll need to configure your pypi credentials via:\n- Configure pypi repository:\n  - `poetry config http-basic.pypi USERNAME PASSWORD`\n- Configure testpypi repository:\n  - `poetry config repositories.testpypi https://test.pypi.org/legacy`\n  - `poetry config http-basic.testpypi USERNAME PASSWORD`\n\n**NOTE:** when running `poetry` commands, particularly `pylint` and `black`, you must `python3.6`\n\n#### Testing\n\nYou can just run `tox` locally to test, lint, and check formatting in the supported python versions. This works by having `tox` manage the virtualenvs, which `poetry` then detects and uses. Note that some of the tools (e.g. `pylint`, `black`) only support running in `python3.6` and this is reflected in `tox.ini`.\n\nRun individual commands can be run interactively in a `poetry shell` session or directly via `poetry run CMD`:\n\n- `pytest` - run python tests\n- `pylint` - lint python code\n- `black .` - format python code\n\n#### Building\n\n- `poetry build` - builds a pip-installable package into `dist/`\n\n#### Releasing\n\n- `poetry version VERSION` - bump the project version (see `poetry version --help` for details)\n- `poetry publish -r testpypi` - publish to testpypi\n- `poetry publish` - publish to pypi\n",
    'author': 'MIT Office of Open Learning',
    'author_email': 'mitx-devops@mit.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/social-auth-mitxpro',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)
